<?php

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Verowa_Templates_List extends WP_List_Table
{
	/** Class constructor */
	public function __construct() {
		parent::__construct( [
			'singular' => __( 'Template', 'verowa-connect' ), //singular name of the listed records
			'plural' => __( 'Templates', 'verowa-connect' ), //plural name of the listed records
			'ajax' => false //should this table support ajax?
		]);

		$this->_column_headers = array(
			 $this->get_columns(),       // columns
			 array(),           // hidden
			 $this->get_sortable_columns(),  // sortable
		);
	}


	public static function get_templates( $per_page = 10, $page_number = 1 ) {
		global $wpdb;

		$query = 'SELECT * FROM `' . $wpdb->prefix . 'verowa_templates`';

		if ( ! empty( $_REQUEST['orderby'] ) ) {
			$query .= ' ORDER BY ' . esc_sql( $_REQUEST['orderby'] );
			$query .= ! empty( $_REQUEST['order'] ) ? ' ' . esc_sql( $_REQUEST['order'] ) : ' ASC';
		}

		$query .= ' LIMIT ' . $per_page;
		$query .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;

		$result = $wpdb->get_results( $query, 'ARRAY_A' );

		return $result;
	}

	public static function delete_template( $id ) {
		global $wpdb;

		$wpdb->delete(
			$wpdb->prefix . 'verowa_templates',
			[ 'template_id' => $id ],
			[ '%d' ]
		);
	}

	public static function record_count() {
		global $wpdb;

		$query = 'SELECT COUNT(*) FROM ' . $wpdb->prefix . 'verowa_templates';

		return $wpdb->get_var( $query );
	}

	/** Text displayed when no template data is available */
	public function no_items() {
		_e( 'No templates avaliable.', 'verowa-connect' );
	}

	/**
	 * Method for name column
	 *
	 * @param array $item an array of DB data
	 *
	 * @return string
	 */
	function column_name( $item ) {

		// create a nonce
		$delete_nonce = wp_create_nonce( 'verowa_delete_template' );

		$title = '<strong>' . $item['name'] . '</strong>';

		$actions = [
			'delete' => sprintf( '<a href="?page=%s&action=%s&template=%s&_wpnonce=%s">Delete</a>',
			esc_attr( $_REQUEST['page'] ), 'delete', absint( $item['template_id'] ), $delete_nonce )
		];

		return $title . $this->row_actions( $actions );
	}

	/**
	 * Render a column when no column specific method exists.
	 *
	 * @param array $item
	 * @param string $column_name
	 *
	 * @return mixed
	 */
	public function column_default( $item, $column_name ) {

		switch ( $column_name ) {
			case 'template_id':
			case 'template_name':
			case 'info_text':
			case 'type':
			case 'display_where':
			case 'header':
			case 'entry':
			case 'separator':
			case 'footer':
			    return $item[ $column_name ];
			case 'edit':
				return '<a href="options-general.php?page=verowa-options-templates&t=' . $item['template_id'] .  '">edit</a>';
			default:
				return print_r( $item, true ); //Show the whole array for troubleshooting purposes
		}
	}

	/**
	 * Render the bulk edit checkbox
	 *
	 * @param array $item
	 *
	 * @return string
	 */
	function column_cb( $item ) {
		return sprintf(
			'<input type="checkbox" name="bulk-delete[]" value="%s" />', $item['template_id']
		);
	}

	/**
	 * Associative array of columns
	 *
	 * @return array
	 */
	function get_columns() {
		$columns = [
			'cb' => '<input type="checkbox" />',
			'template_name' => __( 'Name', 'verowa-connect' ),
			'info_text' => __( 'Info Text', 'verowa-connect' ),
			'type' => __( 'Type', 'verowa-connect' ),
			'display_where' => __( 'Display where', 'verowa-connect' ),
			'header' => __( 'Header', 'verowa-connect' ),
			'separator' => __( 'Separator', 'verowa-connect' ),
			'footer' => __( 'Footer', 'verowa-connect' ),
			'edit' => ''
		];

		return $columns;
	}

	/**
	 * Columns to make sortable.
	 *
	 * @return array
	 */
	public function get_sortable_columns() {
		$sortable_columns = array(
			'template_name' => array( 'template_name', true ),
			'type' => array( 'type', true ),
			'separator' => array( 'separator', false )
		);

		return $sortable_columns;
	}

	/**
	 * Returns an associative array containing the bulk action
	 *
	 * @return array
	 */
	public function get_bulk_actions() {
		$actions = [
			'bulk-delete' => 'Delete'
		];

		return $actions;
	}

	/**
	 * Handles data query and filter, sorting, and pagination.
	 */
	public function prepare_items() {

		$this->_column_headers = $this->get_column_info();

		/** Process bulk action */
		$this->process_bulk_action();

		$per_page = $this->get_items_per_page( 'templates_per_page', 10 );
		$current_page = $this->get_pagenum();
		$total_items = self::record_count();

		$this->set_pagination_args( [
			'total_items' => $total_items, //WE have to calculate the total number of items
			'per_page' => $per_page //WE have to determine how many items to show on a page
		] );

		$this->items = self::get_templates( $per_page, $current_page );
	}

	public function process_bulk_action() {

		//Detect when a bulk action is being triggered...
		if ( 'delete' === $this->current_action() ) {

			// In our file that handles the request, verify the nonce.
			$nonce = esc_attr( $_REQUEST['_wpnonce'] );

			if ( ! wp_verify_nonce( $nonce, 'verowa_delete_template' ) ) {
				die( 'Go get a life script kiddies' );
			}
			else {
				self::delete_template( absint( $_GET['template'] ) );

				wp_redirect( esc_url( add_query_arg() ) );
				exit;
			}

		}

		// If the delete bulk action is triggered
		if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-delete' )
		|| ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-delete' )
		) {

			$delete_ids = esc_sql( $_POST['bulk-delete'] );

			// loop over the array of record IDs and delete them
			foreach ( $delete_ids as $id ) {
				self::delete_template( $id );

			}

			wp_redirect( esc_url( add_query_arg() ) );
			exit;
		}
	}
}

?>